@@ -1,5 +1,5 @@
 #
-#Copyright (C) 2003, 2004, 2005, 2006, 2007, 2009 Olivier Sessink
+#Copyright (C) 2003, 2004, 2005, 2006, 2007, 2009, 2010, 2011 Olivier Sessink
 #All rights reserved.
 #
 #Redistribution and use in source and binary forms, with or without
@@ -65,12 +65,6 @@ def path_is_safe(path, failquiet=0):
 		if (failquiet == 0):
 			sys.stderr.write('ERROR: cannot lstat() '+path+'\n')
 		return -1
-	if (not stat.S_ISDIR(statbuf[stat.ST_MODE])):
-		if (stat.S_ISLNK(statbuf[stat.ST_MODE])):
-			sys.stderr.write('ERROR: '+path+' is a symlink, please point to the real directory\n')
-		else:
-			sys.stderr.write('ERROR: '+path+' is not a directory!\n')
-		return -2
 	if (sys.platform[-3:] == 'bsd'):
 		# on freebsd root is in group wheel
 		if (statbuf[stat.ST_UID] != 0 or statbuf[stat.ST_GID] != grp.getgrnam('wheel').gr_gid):
@@ -80,9 +74,20 @@ def path_is_safe(path, failquiet=0):
 		if (statbuf[stat.ST_UID] != 0 or statbuf[stat.ST_GID] != 0):
 			sys.stderr.write('ERROR: '+path+' is not owned by root:root!\n')
 			return -3
-	if (statbuf[stat.ST_MODE] & stat.S_IWOTH or statbuf[stat.ST_MODE] & stat.S_IWGRP):
+	if ((statbuf[stat.ST_MODE] & stat.S_IWOTH or statbuf[stat.ST_MODE] & stat.S_IWGRP)and not stat.S_ISLNK(statbuf[stat.ST_MODE])):
 		sys.stderr.write('ERROR: '+path+' is writable by group or others!')
 		return -4
+	if (not stat.S_ISDIR(statbuf[stat.ST_MODE])):
+		if (stat.S_ISLNK(statbuf[stat.ST_MODE])):
+			# Fedora has moved /sbin /lib and /bin into /usr
+			target = os.readlink(path)
+			print target, path
+			if (target[:4] != 'usr/'):
+				sys.stderr.write('ERROR: '+path+' is a symlink, please point to the real directory\n')
+				return -2
+		else:
+			sys.stderr.write('ERROR: '+path+' is not a directory!\n')
+			return -2
 	return 1
 
 def chroot_is_safe(path, failquiet=0):
@@ -97,7 +102,7 @@ def chroot_is_safe(path, failquiet=0):
 			return retval
 	npath = os.path.dirname(path)
 	while (npath != '/'):
-		print npath
+		#print npath
 		retval = path_is_safe(npath,0)
 		if (retval != 1):
 #			print 'testing path='+npath+'returned '+str(retval)
@@ -392,9 +397,21 @@ def OLD_create_parent_path(chroot, path, be_verbose=0, copy_permissions=1, allow
 			if (indx==-1):
 				indx=len(directory)
 
+def fix_double_slashes(instring):
+	outstring=''
+	slash=0
+	for i in instring:
+		if (slash==0 or i!= '/'):
+			outstring += i
+		if (i == '/'):
+			slash=1
+		else:
+			slash=0
+	return outstring
+
 def split_path(path):
 	ret = []
-	next=path
+	next=fix_double_slashes(os.path.normpath(path))
 	while (next != '/'):
 		ret.insert(0,os.path.basename(next))
 		next=os.path.dirname(next)
